function fit = maxlik01(fun,beta,x,param);
%
%
%   Computes the MLE assuming independent data.
%   Uses the Fisher method of scoring and a Gauss-Newton type approximation to the Hessian. (The true Hessian is used for the Fisher information matrix.)
%   fun evaluates the loglikelihood at each observation (string)
%   delta is the parameter for numerical derivatives.
%   USASE:  [betanew,varinfo,se] = maxlik(fun,beta,x,string);
%
%   Returns beta = MLE and varinfo = estimated cov. matrix using
%                  an approximate Fisher information matrix.
%                  and se = sqrt(diag(varinfo))
%
%   CALLS:  grad01, hessian01
%
if nargin < 4 ;
    param = '' ;
end ;

if isfield(param,'delta') == 0 ;
    delta = 10e-8 ;
else ;
    delta = param.delta ;   
end ;

if isfield(param,'maxiter') == 0 ;
    maxiter = 200 ;
else ;
    maxiter = param.maxiter ;
end ;

if isfield(param,'tolerance') == 0 ;
    tolerance = .00001 ;
else ;
    tolerance = param.tolerance ;
end ;

if isfield(param,'maxstephalf') == 0 ;
    maxstephalf = 15 ;
else ;
    maxstephalf = param.maxstephalf ;
end ;

if isfield(param,'iprint') == 0 ;
    iprint = 0 ;
else ;
    iprint = param.iprint;   
end ;

m= rows(beta) ;
fdiff = 1 ;        %  fdiff is change in log-likelihood
i =  1 ;           %  i is iteration number

while ( (i < maxiter) & (fdiff > tolerance) ) ;  
    fbase = eval(fun) ;     %  Get current log-likelhood
    n = rows(fbase) ;
    fbase = sum(fbase) ; 
    gl = grad01(fun,beta,x,param) ; 
    h = inv(gl' * gl) ;
    del = h * (ones(1,n)*gl)' ;    %  Direction to increase log-likelihood
    betaold = beta ;
    beta = betaold + del ;         %  New MLE
    fnew =  sum(eval(fun))  ;
    j = 1 ;
    while ( (fnew < fbase) & (j < maxstephalf) ) ;   %  Check is new MLE improves
        del = del/2 ;                %  Half step if new MLE doesn't improve        
        beta = betaold + del ;
        fnew =  sum(eval(fun)) ; 
        j = j + 1 ;
    end ;
    fdiff = fnew - fbase ;      %  Calculate increase in log-likelihood
    i = i + 1 ;
end ;
loglike = fnew ;
varinfo = -inv(hessian01(fun,beta,x,struct('delta',sqrt(delta)))) ;  % inverse Fisher info
se =sqrt(diag(varinfo)) ;
%  print out results
if iprint == 1 ;
    ['number of iterations = ',num2str(i)]
    loglike
    fdiff
    
    'MLE     Std. Errors'
    [beta,se] 
end ;

fit = struct('beta',beta','se',se,'varinfo',varinfo) ;
